#include veil:deferred_utils



uniform sampler2D DiffuseSampler0;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D Noise;
uniform float GameTime;
uniform vec3 heartPos;

in vec2 texCoord;
out vec4 fragColor;

float sphIntersect(vec3 ro, vec3 rd, vec4 sph) {
    vec3 oc = ro - sph.xyz;
    float b = dot(oc, rd);
    float c = dot(oc, oc) - sph.w * sph.w;
    float h = b * b - c;
    if (h < 0.0) {
        return -1.0;
    }
    h = sqrt(h);
    return -b - h;
}
float depthSampleToWorldDepth(float depthSample) {
    float f = depthSample * 2.0 - 1.0;
    return 2.0 * VeilCamera.NearPlane * VeilCamera.FarPlane / (VeilCamera.FarPlane + VeilCamera.NearPlane - f * (VeilCamera.FarPlane - VeilCamera.NearPlane));
}

vec2 boxIntersection(in vec3 ro, in vec3 rd, vec3 boxSize, vec3 outNormal) {
    vec3 m = 1.0 / rd;  // can precompute if traversing a set of aligned boxes
    vec3 n = m * ro;    // can precompute if traversing a set of aligned boxes
    vec3 k = abs(m) * boxSize;
    vec3 t1 = -n - k;
    vec3 t2 = -n + k;
    float tN = max(max(t1.x, t1.y), t1.z);
    float tF = min(min(t2.x, t2.y), t2.z);
    if (tN > tF || tF < 0.0) return vec2(-1.0);  // no intersection
    outNormal = (tN > 0.0) ? step(vec3(tN), t1) :  // ro ouside the box
    step(t2, vec3(tF));                  // ro inside the box
    outNormal *= -sign(rd);
    return vec2(tN, tF);
}
vec3 viewPosFromDepth(float depth, vec2 uv) {
    float z = depth * 2.0 - 1.0;

    vec4 positionCS = vec4(uv * 2.0 - 1.0, z, 1.0);
    vec4 positionVS = VeilCamera.IProjMat * positionCS;
    positionVS /= positionVS.w;

    return positionVS.xyz;
}

vec3 viewDirFromUv(vec2 uv) {
    return (VeilCamera.IViewMat * vec4(normalize(viewPosFromDepth(1.0, uv)), 0.0)).xyz;
}

float hash21(vec2 p) {
    p = fract(p * vec2(123.34, 456.78));
    p += dot(p, p + 34.45);
    return fract(p.x * p.y);
}


void main() {
    vec4 baseColor = texture(DiffuseSampler0, texCoord);
    float depthSample = texture(DiffuseDepthSampler, texCoord).r;
    vec3 viewPos = viewPosFromDepthSample(depthSample, texCoord);
    vec3 rayDir = viewDirFromUv(texCoord);

    int numPillars = 21;
    float randomSeed = 69420;
    float dispersion = 50;
    float dist;

    for (int i = 0; i < numPillars; ++i) {
        dist = length(viewPos);

        float randomX = (hash21(vec2(float(i), randomSeed))) * dispersion - dispersion/ 2;
        float randomZ = (hash21(vec2(float(i) + 1.0 , randomSeed + 10.0))) * dispersion - dispersion / 2;

        vec3 pillarPositionOffset = vec3(randomX, 140, randomZ);
        vec3 boxPosition = VeilCamera.CameraPosition - pillarPositionOffset + vec3(-heartPos.x, -tan(GameTime * 1000.0 * hash21(vec2(float(i), randomSeed))) * 10.0 + 100.0, -heartPos.z);
        vec2 rayHit = boxIntersection(boxPosition, rayDir, vec3(0.2, 4.0, 0.2), vec3(0.0, 0.0, 1.0));

        if (rayHit.x > 0.0 && dist / 10 > 1) {
            fragColor = vec4(vec3(1.0, 1.0, 0.8) , 1.0);
            return;
        }
    }

    fragColor = vec4(vec3(baseColor), 1.0);
}